﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Markup;
using System.Windows.Media;

namespace Wpf.CommonTools.SystemExtension
{
    /// <summary>
    /// Adorner for the watermark
    /// </summary>
    public class WatermarkAdorner : Adorner
    {
        #region Private Fields

        /// <summary>
        /// <see cref="ContentPresenter"/> that holds the watermark
        /// </summary>
        private readonly ContentPresenter contentPresenter;

        #endregion

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the <see cref="WatermarkAdorner"/> class
        /// </summary>
        /// <param name="adornedElement"><see cref="UIElement"/> to be adorned</param>
        /// <param name="watermark">The watermark</param>
        public WatermarkAdorner(UIElement adornedElement, object watermark) :
            base(adornedElement)
        {
            this.IsHitTestVisible = false;

            this.contentPresenter = new ContentPresenter();
            this.contentPresenter.Content = watermark;

            if (this.Control is ComboBox)
            {
                //this.contentPresenter.VerticalAlignment = VerticalAlignment.Center;
                //this.contentPresenter.HorizontalAlignment = HorizontalAlignment.Center;
                this.contentPresenter.Opacity = 0.5;
                this.contentPresenter.Margin = new Thickness(
                    Control.Margin.Left + Control.Padding.Left + Control.BorderThickness.Left + 1,
                    Control.Margin.Top + Control.Padding.Top - 3,
                    Control.Margin.Right + Control.Padding.Right + Control.BorderThickness.Right + 7,
                    Control.Margin.Bottom + Control.Padding.Bottom-2);
            }
            else
            {
                this.contentPresenter.Opacity = 0.3;
                this.contentPresenter.Margin = new Thickness(
                    Control.Margin.Left + Control.Padding.Left + Control.BorderThickness.Left + 1,
                    Control.Margin.Top + Control.Padding.Top + Control.BorderThickness.Top,
                    Control.Margin.Right + Control.Padding.Right + Control.BorderThickness.Right + 2,
                    Control.Margin.Bottom + Control.Padding.Bottom);
            }
            //if (watermark as TextBlock != null)
            //{
            //  TextBlock tb=  ((TextBlock)watermark);
            //  Size a = Measure(tb.Text, tb.FontSize, tb.FontStyle.ToString());
            //  this.Control.Padding = new Thickness(this.Control.Padding.Left, this.Control.Padding.Top, this.Control.Padding.Right + a.Width, this.Control.Padding.Bottom);
            //}

            // Hide the control adorner when the adorned element is hidden
            Binding binding = new Binding("IsVisible");
            binding.Source = adornedElement;
            binding.Converter = new BooleanToVisibilityConverter();
            this.SetBinding(VisibilityProperty, binding);
        }

        public Size Measure(string text, double fontSize, string typeFace)
        {
            FormattedText ft = new FormattedText(text,
              System.Globalization.CultureInfo.CurrentCulture,
              FlowDirection.LeftToRight,
              new Typeface(typeFace),
              fontSize,
              Brushes.Black);
            return new Size(ft.Width, ft.Height);
        }

        #endregion

        #region Protected Properties

        /// <summary>
        /// Gets the number of children for the <see cref="ContainerVisual"/>.
        /// </summary>
        protected override int VisualChildrenCount
        {
            get { return 1; }
        }

        #endregion

        #region Private Properties

        /// <summary>
        /// Gets the control that is being adorned
        /// </summary>
        private Control Control
        {
            get { return (Control)this.AdornedElement; }
        }

        #endregion

        #region Protected Overrides

        /// <summary>
        /// Returns a specified child <see cref="Visual"/> for the parent <see cref="ContainerVisual"/>.
        /// </summary>
        /// <param name="index">A 32-bit signed integer that represents the index value of the child <see cref="Visual"/>. The value of index must be between 0 and <see cref="VisualChildrenCount"/> - 1.</param>
        /// <returns>The child <see cref="Visual"/>.</returns>
        protected override Visual GetVisualChild(int index)
        {
            return this.contentPresenter;
        }

        /// <summary>
        /// Implements any custom measuring behavior for the adorner.
        /// </summary>
        /// <param name="constraint">A size to constrain the adorner to.</param>
        /// <returns>A <see cref="Size"/> object representing the amount of layout space needed by the adorner.</returns>
        protected override Size MeasureOverride(Size constraint)
        {
            // Here's the secret to getting the adorner to cover the whole control
            this.contentPresenter.Measure(Control.RenderSize);
            return Control.RenderSize;
        }

        /// <summary>
        /// When overridden in a derived class, positions child elements and determines a size for a <see cref="FrameworkElement"/> derived class. 
        /// </summary>
        /// <param name="finalSize">The final area within the parent that this element should use to arrange itself and its children.</param>
        /// <returns>The actual size used.</returns>
        protected override Size ArrangeOverride(Size finalSize)
        {
            this.contentPresenter.Arrange(new Rect(finalSize));
            return finalSize;
        }

        #endregion
    }
}
